package com.example.webapp.repository;

import com.example.webapp.entity.Contract;
import com.tsurugidb.iceaxe.session.TsurugiSession;
import com.tsurugidb.iceaxe.sql.TsurugiSqlPreparedQuery;
import com.tsurugidb.iceaxe.sql.TsurugiSqlPreparedStatement;
import com.tsurugidb.iceaxe.sql.TsurugiSqlQuery;
import com.tsurugidb.iceaxe.sql.parameter.TgBindParameters;
import com.tsurugidb.iceaxe.sql.parameter.TgBindVariables;
import com.tsurugidb.iceaxe.sql.parameter.TgParameterMapping;
import com.tsurugidb.iceaxe.sql.result.TgResultMapping;
import com.tsurugidb.iceaxe.transaction.TsurugiTransaction;
import com.tsurugidb.iceaxe.transaction.exception.TsurugiTransactionException;
import org.springframework.stereotype.Repository;

import java.io.IOException;
import java.time.LocalDate;
import java.util.List;
import java.util.Optional;

@Repository
public class ContractRepository {

    private static final TgResultMapping<Contract> RESULT_MAPPING =
            TgResultMapping.of(Contract::new)
                    .addString("phone_number", Contract::setPhoneNumber)
                    .addDate("start_date", Contract::setStartDate)
                    .addDate("end_date", Contract::setEndDate)
                    .addString("charge_rule", Contract::setChargeRule);

    public List<Contract> findAll(TsurugiTransaction transaction)
            throws TsurugiTransactionException, IOException, InterruptedException {
        TsurugiSession session = transaction.getSession();
        var sqlStr = "select * from contracts";
        try (TsurugiSqlQuery<Contract> query = session.createQuery(sqlStr, RESULT_MAPPING)) {
            return transaction.executeAndGetList(query);
        }
    }

    public Optional<Contract> findByPk(TsurugiTransaction transaction, String phoneNumber, LocalDate startDate)
            throws TsurugiTransactionException, IOException, InterruptedException {
        TsurugiSession session = transaction.getSession();
        var sqlStr = "select * from contracts where phone_number = :phone_number and start_date = :start_date";
        TgBindVariables variables = TgBindVariables.of().addString("phone_number").addDate("start_date");
        TgParameterMapping<TgBindParameters> mapping = TgParameterMapping.of(variables);
        try (TsurugiSqlPreparedQuery<TgBindParameters, Contract> query = session.createQuery(sqlStr, mapping,
                RESULT_MAPPING)) {
            TgBindParameters params = TgBindParameters.of().addString("phone_number", phoneNumber)
                    .addDate("start_date", startDate);
            return transaction.executeAndFindRecord(query, params);
        }
    }

    public void create(TsurugiTransaction transaction, Contract contract)
            throws IOException, InterruptedException, TsurugiTransactionException {
        TsurugiSession session = transaction.getSession();
        var sqlStr = """
                insert into contracts (phone_number, start_date, end_date, charge_rule)
                values (:phone_number, :start_date, :end_date, :charge_rule)
                """;
        TgBindVariables variables = TgBindVariables.of().addString("phone_number").addDate("start_date")
                .addDate("end_date").addString("charge_rule");
        TgParameterMapping<TgBindParameters> mapping = TgParameterMapping.of(variables);
        try (TsurugiSqlPreparedStatement<TgBindParameters> stmt = session.createStatement(sqlStr, mapping)) {
            TgBindParameters params = TgBindParameters.of()
                    .addString("phone_number", contract.getPhoneNumber())
                    .addDate("start_date", contract.getStartDate())
                    .addDate("end_date", contract.getEndDate())
                    .addString("charge_rule", contract.getChargeRule());
            transaction.executeAndGetCount(stmt, params);
        }
    }

    public void update(TsurugiTransaction transaction, Contract contract)
            throws TsurugiTransactionException, IOException, InterruptedException {
        TsurugiSession session = transaction.getSession();
        var sqlStr = """
                update contracts set end_date = :end_date, charge_rule = :charge_rule
                where phone_number = :phone_number and start_date = :start_date
                """;
        TgBindVariables variables = TgBindVariables.of().addDate("end_date").addString("charge_rule")
                .addString("phone_number").addDate("start_date");
        TgParameterMapping<TgBindParameters> mapping = TgParameterMapping.of(variables);
        try (TsurugiSqlPreparedStatement<TgBindParameters> stmt = session.createStatement(sqlStr, mapping)) {
            TgBindParameters params = TgBindParameters.of()
                    .addString("phone_number", contract.getPhoneNumber())
                    .addDate("start_date", contract.getStartDate())
                    .addDate("end_date", contract.getEndDate())
                    .addString("charge_rule", contract.getChargeRule());
            transaction.executeAndGetCount(stmt, params);
        }
    }

    public void deleteByPk(TsurugiTransaction transaction, String phoneNumber, LocalDate startDate)
            throws TsurugiTransactionException {
        TsurugiSession session = transaction.getSession();
        var sqlStr = "delete from contracts where phone_number = :phone_number and start_date = :start_date";
        TgBindVariables variables = TgBindVariables.of().addString("phone_number").addDate("start_date");
        TgParameterMapping<TgBindParameters> mapping = TgParameterMapping.of(variables);
        try (TsurugiSqlPreparedStatement<TgBindParameters> stmt = session.createStatement(sqlStr, mapping)) {
            TgBindParameters params = TgBindParameters.of()
                    .addString("phone_number", phoneNumber)
                    .addDate("start_date", startDate);
            transaction.executeAndGetCount(stmt, params);
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

}
