package com.example.webapp.service;

import com.example.webapp.dto.ContractInput;
import com.example.webapp.entity.Contract;
import com.example.webapp.repository.ContractRepository;
import com.tsurugidb.iceaxe.TsurugiConnector;
import com.tsurugidb.iceaxe.exception.TsurugiExceptionUtil;
import com.tsurugidb.iceaxe.exception.TsurugiIOException;
import com.tsurugidb.iceaxe.session.TsurugiSession;
import com.tsurugidb.iceaxe.transaction.manager.TgTmSetting;
import com.tsurugidb.iceaxe.transaction.manager.TsurugiTransactionManager;
import com.tsurugidb.iceaxe.transaction.option.TgTxOption;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.time.LocalDate;
import java.util.List;
import java.util.Optional;

@Service public class ContractService {

    private final TsurugiConnector tsurugiConnector;

    private final ContractRepository contractRepository;

    public ContractService(TsurugiConnector tsurugiConnector, ContractRepository contractRepository) {
        this.tsurugiConnector = tsurugiConnector;
        this.contractRepository = contractRepository;
    }

    public List<Contract> findAll() {
        try (TsurugiSession session = tsurugiConnector.createSession()) {
            TsurugiTransactionManager tm = session.createTransactionManager(TgTxOption.ofRTX());
            return tm.execute(transaction -> {
                return contractRepository.findAll(transaction);
            });
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public Optional<String> create(ContractInput contractInput) {
        try (TsurugiSession session = tsurugiConnector.createSession()) {
            TsurugiTransactionManager tm = session.createTransactionManager(
                    TgTmSetting.ofAlways(TgTxOption.ofOCC(), 1));
            return tm.execute(transaction -> {
                if (contractInput.getEndDate() != null && contractInput.getStartDate()
                        .isAfter(contractInput.getEndDate())) {
                    return Optional.of("契約終了日は契約開始日より後に設定してください");
                }
                Contract newContract = new Contract(contractInput.getPhoneNumber(), contractInput.getStartDate(),
                        contractInput.getEndDate(), contractInput.getChargeRule());
                contractRepository.create(transaction, newContract);
                return Optional.empty();
            });
        } catch (TsurugiIOException e) {
            if (TsurugiExceptionUtil.getInstance().isUniqueConstraintViolation(e)) {
                return Optional.of("その契約情報は既に存在します");
            } else {
                throw new RuntimeException(e);
            }
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public Optional<Contract> findOne(String phoneNumber, LocalDate startDate) {
        try (TsurugiSession session = tsurugiConnector.createSession()) {
            TsurugiTransactionManager tm = session.createTransactionManager(TgTxOption.ofRTX());
            return tm.execute(transaction -> {
                return contractRepository.findByPk(transaction, phoneNumber, startDate);
            });
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public Optional<String> update(ContractInput contractInput) {
        try (TsurugiSession session = tsurugiConnector.createSession()) {
            TsurugiTransactionManager tm = session.createTransactionManager(
                    TgTmSetting.ofAlways(TgTxOption.ofOCC(), 1));
            return tm.execute(transaction -> {
                if (contractInput.getEndDate() != null && contractInput.getStartDate()
                        .isAfter(contractInput.getEndDate())) {
                    return Optional.of("契約終了日は契約開始日より後に設定してください");
                }
                Contract contract = new Contract(contractInput.getPhoneNumber(), contractInput.getStartDate(),
                        contractInput.getEndDate(), contractInput.getChargeRule());
                contractRepository.update(transaction, contract);
                return Optional.empty();
            });
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public void delete(String phoneNumber, LocalDate startDate) {
        try (TsurugiSession session = tsurugiConnector.createSession()) {
            TsurugiTransactionManager tm = session.createTransactionManager(
                    TgTmSetting.ofAlways(TgTxOption.ofOCC(), 1));
            tm.execute(transaction -> {
                contractRepository.deleteByPk(transaction, phoneNumber, startDate);
            });
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

}
